* Program to compute the two-loop 
* beta function, for SU(N) YM, 
* based on the diagrammatic
* expression in hep-th/0508026 v2
* by  Oliver J. Rosten

**********************************************************************************

CF De;	* Effective Propagator object.
CF v;	* Vertex object.
CF GR;	* GR object.
CF Combo; * a combination of GRs and (pseudo) effective propagators .

CF i;	* 1/(square inverse). With two mom arguments of form (u,-v) - expand in powers of the second.

CF box;	* The usual definition

* ******************************* Dimensionality **********************************

S ep;		* Dimension is 4 - 2*ep
S em;		* Euler-Mascheroni constant
S OmegaD;	* Area of unit sphere in D dimensions
S AngD; 	* equals 1/(4*pi)^(D/2)
S [d];
D [d];

* ******************************* Vectors, Indices and Tensors ******************

V k,p,l;
* Program to compute beta_2 from the revised version of hep-th/0508026
* by  Oliver J. Rosten

**********************************************************************************


I mu,nu,al,be,ga,de,si,rh,ta;
T t,u;

* mu and nu are external indices; all others are internal

index al=0;
index be=0;
index ga=0;
index de=0;
index rh=0;
index si=0;
index ta=0;

* ******************************** Various Symbols ******************************
S pw;	* Tag for pseudo wine
S ps,ng;* Tag for derivatives of GRs: for GR carrying momentum k, ps / ng --> deriv wrt +-k
S NF;	* Tags for non-factorizable integrals
S K,L;	* Counting parameters for degree of IR divergence in k and l integrals
S X;	* local counter for instances of i(l-k)
S x,y;	* i^-1
S xep;	* i^ep
S N;	* as in SU(N); all contributions go as N^2

* ******************************** Feynman Parameterisation **********************

S a,b;	* Feynman parameters.
S den;
S den1;	* Generic denominator: used at intermediate stages of Feynman integration
S den2;

* ******************************** Generic Counters ******************************

S n,m,r;

* ******************************** Diagrams **************************************

L [T.8]	= -De(k)*De(l-k)*De(l,-p)*v(0,mu,al,be,ga)*v(0,nu,ga,be,al);

L [T.26] 	= De(l)*De(l,-p)*De(l-k)*De(k)*
				(v(l,mu,al,be) + v(p,be,mu,al))*v(0,al,nu,rh,si)*
				(
					v(l,rh,be,si) + v(k,be,si,rh) +
					-v(l,si,rh,be) - v(k,be,si,rh)
				);

L [T.27]	= De(l)*De(l,-p)*De(l-k)*De(k,-p)*
				(v(l,mu,al,be) + v(p,be,mu,al))*v(0,be,si,rh,nu)*
				(
					v(p,si,al,rh) + v(k,al,rh,si) + v(l,rh,si,al)
					-v(p,rh,si,al) - v(k,al,rh,si) - v(l,si,al,rh)
				);

L [T.28]	= De(l)*De(l,-p)*De(k)*De(k,-p)*1/2*
				(v(l,mu,al,be) + v(p,be,mu,al))*v(0,al,rh,si,be)*
				(
					v(p,si,rh,nu) + v(k,nu,si,rh)
					-v(p,rh,nu,si) - v(k,nu,si,rh)
				);

L [T.29]	= -De(l)*De(l,-p)*De(l-k)*De(k,-p)*De(k)*
				(v(l,mu,al,be) + v(p,be,mu,al))*
				(
				(v(p,ta,al,si) + v(k,al,si,ta) + v(l,si,ta,al))*
				(v(k,be,ta,rh) + v(l,rh,be,ta))*
				(v(p,rh,si,nu) + v(k,nu,rh,si))
				);

L [T.30]	= 2*De(l,-p)*De(l)*(v(l,mu,al,be) + v(p,be,mu,al))*
				GR(k,be)*GR(k-l,al)*GR(k,-p,nu);


L [T.31]	= -2*De(l,-p)*De(l)*(v(l,mu,al,be) + v(p,be,mu,al))*
				GR(-k,nu)*GR(l-k,be)*GR(-k,p,al);

L [T.32]	= 4*De(k)*GR(l,-p,nu)*GR(l,mu)*GR(l-k,al)*GR(l,al);

L [T.33]	= 4*De(l-k)*GR(l,-p,nu)*GR(l,mu)*GR(k,al)*GR(l,al);

L [T.34]	= 2*De(l-k)*GR(l,al)*GR(l,-p,nu)*GR(k,-p,al)*GR(k,mu);


L [T.35]	= 8*box(p,mu,al)*De(l)*De(l-k)*Combo(k,ga,nu)*
				(v(k,al,be,ga) + v(l,ga,al,be))*GR(l,be);


L [T.36]	= -8*box(p,mu,al)*GR(l-k,al)*GR(l,be)*De(l)*Combo(-k,be,nu);


L [T.37]	= -8*box(p,mu,al)*De(l-k)*GR(k,be)*GR(l,be)*Combo(l,al,nu);

L [T.38]	= -8*box(p,mu,al)*De(k)*GR(l-k,be)*GR(l,be)*Combo(l,al,nu);

L [T.39]	= 8*box(p,mu,al)*Combo(k,al,be)*Combo(l,be,nu);



L [T.40]	= -De(l)*De(l,-p)*(v(l,mu,al,be) + v(p,be,mu,al))*
				(
					De(k)*De(l-k)*
						(v(k,al,rh,si) + v(l,rh,si,al))*v(0,k,nu,k,be,si,rh)
					+
					De(k)*De(l-k)*
						v(0,k,nu,k,al,rh,si)*(v(k,be,si,rh) + v(l,rh,be,si))


					+(De(k,ps,nu)*De(l-k))*2*
					(v(k,be,si,rh) + v(l,rh,be,si))*(v(k,al,rh,si) + v(l,rh,si,al))


					-2*GR(k,be,ps,nu)*GR(k-l,al)
					-2*GR(-k,al,ng,nu)*GR(l-k,be)

					+2*De(k)*De(l-k)*
					(
					v(0,l,nu,l,rh,be,si)*(v(k,al,rh,si) + v(l,rh,si,al))
					+(v(k,be,si,rh) + v(l,rh,be,si))*v(0,l,nu,l,rh,si,al)
					)
				);


L [T.41]	= -De(l)*De(l,-p)*(v(l,mu,al,be) + v(p,be,mu,al))*
				(
					De(k)*De(k,-p)*
						v(0,k,be,k,nu,si,rh)*(v(p,si,al,rh) + v(k,al,rh,si))
					+
					De(k)*De(k,-p)*
						(v(p,si,rh,nu) + v(k,nu,si,rh))*v(0,k,be,k,al,rh,si)


					+(De(k,ps,be)*De(k,-p))*2*
					(v(p,rh,si,nu) + v(k,nu,rh,si))*(v(p,rh,al,si) + v(k,al,si,rh))


					-2*GR(k,al,ps,be)*GR(k,-p,nu)
					-2*GR(-k,nu,ng,be)*GR(-k,p,al)

					+2*De(k)*De(k,-p)*
					(
					v(0,p,be,p,si,al,rh)*(v(p,si,rh,nu) + v(k,nu,si,rh))
					+(v(p,si,al,rh) + v(k,al,rh,si))*v(0,p,be,p,si,rh,nu)
					)
				);






L [T.42]	= 	1/2*De(l)^2*De(l,-p)*2*		
				( v(l,mu,al,be) + v(p,be,mu,al) )*
				( v(l,nu,ta,al) + v(p,ta,al,nu) )*
				(
					(
						20/3*1/ep  - (20/3*(1/ep-em) + 124/9)*i(l)^ep
					)*box(l,be,ta) 
				)*AngD;	 



L [T.43]	= 	4*box(p,mu,be)*De(l)^2*GR(l,ta,ps,nu)*
				(
					(20/3*1/ep - (20/3*(1/ep-em) + 124/9)*i(l)^ep)*
					box(l,be,ta)
				)*AngD;	

L [T.44] 	=	-De(l)*De(l,-p)*(v(l,mu,al,be) + v(p,be,mu,al))*
				(
					v(l,nu,be,al)/2*(20/3*1/ep - (20/3*(1/ep-em) + 124/9)*i(l)^ep)
					+2*i(l)^ep*l(nu)*i(l)*box(l,al,be)*20/3 
				)*AngD;


L [T.45]	=	-De(l)*De(l,-p)*(v(l,mu,al,be) + v(p,be,mu,al))*
				(
					20/3*1/ep*v(p,be,al,nu)/2
				)*AngD;


L [T.46-49] =	-2*box(p,mu,al)*Combo(l,al,nu)*20/3*1/ep*AngD;


L [-4*beta_2*box(p,mu,nu)]	= 
			  [T.8] + [T.26] + [T.27] + [T.28] + [T.29] + [T.30] + [T.31] 
			+ [T.32] + [T.33]+ [T.34] + [T.35] + [T.36] 
			+ [T.37] + [T.38] + [T.39] + [T.40] + [T.41] + [T.42]
			+ [T.43] + [T.44] + [T.45] +  [T.46-49];


* ******************************** Program ***************************************

multiply (4*ep*N^2);	* All diagrams go as N^2; the 4*ep is from differentiation wrt -Lambda dLambda|_alpha

* Expand out Gauge Remainders, Effective Propagators and Vertices

#include GR.frm
#include effprops.frm
#include vertices.frm

#include dummy_sum.frm
id i(l-k) = i(l-k)*NF;

#call expand(l,p)
#call expand(k,p)

id l.l*i(l)=1;
id k.k*i(k)=1;
.sort

#call order(p,2)

if (match(NF));
	#include organise.frm
	#include Feynman.frm
else;
	#call averageD(l);
	id l.l*i(l) = 1;
	#call Integral(l,y);
endif;
id NF=1;

#call averageD(k)
id k.k*i(k) = 1;
id l.l*i(l) = 1;

id p.p*d_(mu,nu) = box(p,mu,nu) + p(mu)*p(nu);
id p.p*i(p) = 1;
.sort

id i(k)= i(k)*K;
if (match(K));
#call Integral(k,x)
endif;
id K=1;

id OmegaD = 2*AngD*invfac_(1-ep);
#include gammasD.frm
#include dimensionD.frm


if (count(ep,1)>0) discard;


B ep;
P +s;
.end
